@@ -2,7 +2,7 @@
 
   error.c -
 
-  $Author$
+  $Author: nagachika $
   created at: Mon Aug  9 16:11:34 JST 1993
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
