@@ -2,7 +2,7 @@
 
   ruby.c -
 
-  $Author$
+  $Author: usa $
   created at: Tue Aug 10 12:47:31 JST 1993
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
@@ -369,7 +369,9 @@ translit_char_bin(char *p, int from, int to)
 	p++;
     }
 }
+#endif
 
+#ifdef _WIN32
 # define UTF8_PATH 1
 #endif
 
@@ -1075,7 +1077,7 @@ proc_options(long argc, char **argv, struct cmdline_options *opt, int envopt)
 		if (v > 0377)
 		    rb_rs = Qnil;
 		else if (v == 0 && numlen >= 2) {
-		    rb_rs = rb_str_new2("\n\n");
+		    rb_rs = rb_str_new2("");
 		}
 		else {
 		    c = v & 0xff;
@@ -1366,6 +1368,9 @@ process_options(int argc, char **argv, struct cmdline_options *opt)
     VALUE parser;
     const rb_iseq_t *iseq;
     rb_encoding *enc, *lenc;
+#if UTF8_PATH
+    rb_encoding *uenc, *ienc = 0;
+#endif
     const char *s;
     char fbuf[MAXPATHLEN];
     int i = (int)proc_options(argc, argv, opt, 0);
@@ -1476,6 +1481,9 @@ process_options(int argc, char **argv, struct cmdline_options *opt)
 	enc = rb_enc_from_index(opt->intern.enc.index);
 	rb_enc_set_default_internal(rb_enc_from_encoding(enc));
 	opt->intern.enc.index = -1;
+#if UTF8_PATH
+	ienc = enc;
+#endif
     }
     rb_enc_associate(opt->script_name, lenc);
     rb_obj_freeze(opt->script_name);
@@ -1510,8 +1518,11 @@ process_options(int argc, char **argv, struct cmdline_options *opt)
 #undef SET_COMPILE_OPTION
     }
 #if UTF8_PATH
-    opt->script_name = str_conv_enc(opt->script_name, rb_utf8_encoding(), lenc);
-    opt->script = RSTRING_PTR(opt->script_name);
+    uenc = rb_utf8_encoding();
+    if (uenc != lenc) {
+	opt->script_name = str_conv_enc(opt->script_name, uenc, lenc);
+	opt->script = RSTRING_PTR(opt->script_name);
+    }
 #endif
     ruby_set_argv(argc, argv);
     process_sflag(&opt->sflag);
@@ -1535,7 +1546,15 @@ process_options(int argc, char **argv, struct cmdline_options *opt)
 	}
 	else {
 	    eenc = lenc;
+#if UTF8_PATH
+	    if (ienc) eenc = ienc;
+#endif
 	}
+#if UTF8_PATH
+	if (eenc != uenc) {
+	    opt->e_script = str_conv_enc(opt->e_script, uenc, eenc);
+	}
+#endif
 	rb_enc_associate(opt->e_script, eenc);
 	if (!(opt->dump & ~DUMP_BIT(version_v))) {
 	    ruby_set_script_name(opt->script_name);
